from django import template
from .. import report_lists

register = template.Library()

@register.simple_tag(takes_context=True)
def available_reports(context):
    reports = [(report.name, report.get_verbose_name(), report.get_form()) for report in report_lists]
    reports.insert(0, (None, "Select a report", None))
    context['available_reports'] = reports
    return ''



from unittest import TestCase
from hamcrest import *
from report import HTSQLReport, Report
from report_handler import MemoryReportHandler, DjangoReportHandler
from report_tracking.models import ReportTracking

class MyReportTest(HTSQLReport):
        encoding = "latin-1"
        query = "/school"
        delimiter = ";"
        connexion = "sqlite:htsql_demo.sqlite"
        name = "myReport"
        arguments = ['name']

class TestReport(TestCase):
    def test_synch_report(self):
        report = MyReport()
        result = report.produce()
        content = report.get_data()
        file_report = report.as_file("/tmp/report.csv")
        assert_that(len(content.splitlines()), is_(9))
        assert_that(";" in content)
        assert_that(isinstance(file_report, file))

    def test_asynch_report(self):
        report = MyReport()
        report.asynchronous = True
        result = report.produce()
        content = report.get_data()
        assert_that(report.status(), is_not(-1))
        assert_that(len(content.splitlines()), is_(9))

    def test_report_status(self):
        report_handler = MemoryReportHandler()
        report = MyReport(report_handler=report_handler)

        assert_that(len(report_handler.get_all_reports()), is_(1))
        assert_that(report_handler.get_all_reports()[0][0], none())

        report.produce()
        assert_that(report_handler.get_all_reports()[0][0], not_none())
        assert_that(report_handler.get_all_reports()[0][2], is_(Report.DONE))

class DjangoReportHandlerTest(TestCase):

    def test_report_add_entries(self):
        report_handler = DjangoReportHandler()
        report = MyReport(report_handler=report_handler)
        report2 = MyReport(report_handler=report_handler)

        report.produce()
        assert_that(ReportTracking.objects.count(), is_(1))
        assert_that(ReportTracking.objects.all()[0].status, is_(Report.DONE))



from django.conf.urls.defaults import patterns, include, url
from report_tracking.views import ReportTrackingDetailView, ReportTrackingView

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'boot.views.home', name='home'),
     url(r'^reports/', ReportTrackingView.as_view(), name="report-lists"),
     url(r'^report/(?P<pk>\d+)/$', ReportTrackingDetailView.as_view(), name="report-download"),

    # Uncomment the admin/doc line below to enable admin documentation:
    # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
     url(r'^admin/', include(admin.site.urls)),
)



report_lists = set()

def register(report_class):
    if report_class in report_lists:
        raise ValueError("Already registered report class %s" % report_class)
    report_lists.add(report_class) 

def get_report_by_name(name):
    report = filter(lambda report_class: report_class.name == name, report_lists)
    if report:
        return report[0]
    raise ValueError("No report %s registered" % name)



import os
from django.db import models
from django.contrib.auth.models import User
from . import report_lists
from jsonfield import JSONField

# Create your models here.
class ReportTracking(models.Model):

    report_name = models.CharField(max_length=100)
    status = models.IntegerField(default=0)
    report_date = models.DateTimeField()
    report_file = models.FilePathField(null=True)
    user = models.ForeignKey(User, blank=True, null=True)
    data = JSONField(null=True)

    class Meta:
        ordering = ["-report_date"]


    @classmethod
    def report_lists(cls):
        return [report.name for report in report_lists]

    def display_status(self):
        from report import Report
        return Report.status_description.get(self.status, "Invalid status") 

    def is_file_ready(self):
       from report import Report
       return self.status == Report.DONE and os.path.exists(self.report_file)

    @property
    def download(self):
       return "download"

    @property
    def parameters(self):
        params = []
        for key in self.data:
            params.append( "%s:%s" % (key, self.data[key]))
        return "|".join(params)

    @models.permalink
    def get_absolute_url(self):
        return ('report-download', (), {'pk':self.pk})



"""Special settings file for use when testing.  This specifies a SQLite
database to use when running tests.

Just make sure you run the tests and specify this file:
> ./manage.py test --settings=test_settings
"""

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', 
		'NAME': ':memory:',
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'

INSTALLED_APPS = ('report_tracking',)



from datetime import datetime

class ReportHandler(object):
    def __init__(self):
        self.reports_exec = {}

    def add_report(self, report):
        self.reports_exec[report] = (None, report, None)
    
    def pre_run(self, report, **kwargs):
        self.reports_exec[report] = (datetime.now(), report, None)

    def post_run(self, report, **kwargs):
        self.update_status(report)

    def update_status(self, report):
        date, report, status = self.reports_exec[report]
        self.reports_exec[report] = (date, report, report.status())

    def get_all_reports(self):
        for report in self.reports_exec:
            self.update_status(report)
        return self.reports_exec.values()

class MemoryReportHandler(ReportHandler):    
    pass



#!/usr/bin/env python
import os

from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='django-async-reports',
      version='0.1.8',
      long_description =read('README.txt'),
      description='Reporting for django',
      keywords = "CSV Django reporting",
      author='Anthony TRESONTANI',
      author_email='dev.tresontani@gmail.com',
      include_package_data=True,
      packages=find_packages(exclude=["async_reports.test_app"]),
      install_requires=[
          'Django==1.3.1',
          'South==0.7.3',
          'django-jsonfield==0.8.10',
          'celery==2.5.3',
          ],
      classifiers=['Environment :: Web Environment',
                   'Development Status :: 2 - Pre-Alpha',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: Unix',
                   'Programming Language :: Python']
)



from async_messages import message_user
from async_reports.report_handler import ReportHandler
from .models import ReportTracking

class DjangoReportHandler(ReportHandler):
   def pre_run(self, report, **kwargs):
        super(DjangoReportHandler, self).pre_run(report)
        date, report, status = self.reports_exec[report]
        self.report = report
        self.report_tracking, created = ReportTracking.objects.get_or_create(report_name=self.report.name,status=self.report.status(), report_date=date)

   def post_run(self, report, **kwargs):
       super(DjangoReportHandler, self).post_run(report)

       user = kwargs.get("user", None)
       self.report_tracking.status = report.status()
       self.report_tracking.report_file = self.report.filename
       self.report_tracking.data = self.report.parameters
       self.report_tracking.user = user
       self.report_tracking.save()

       if self.report_tracking.user:
           url = self.report_tracking.get_absolute_url()
           message_user(self.report_tracking.user, "Your report %s is ready <a href='%s'>here</a>" % (self.report.name, url))



import StringIO
import datetime
from htsql import HTSQL
from celery.task import task

from report_handler import ReportHandler
from formatter import CSVFormatter

from django.conf import settings

class ReportMetaClass(type):
    def __new__(meta, classname, bases, classDict):
        cls = type.__new__(meta, classname, bases, classDict)
        if not classname in ["Report", "HTSQLReport"]:
            if not "name" in classDict:
                raise AttributeError("%s class should have a name" % classname)
        return cls


class Report(object):

    __metaclass__ = ReportMetaClass

    NO_RUN, RUNNING, DONE, FAILED = 0, 1, 2, -1
    status_description = {0: "No run",
                          1: "Running",
                          2: "Done",
                          -1: "Failed",
                          }

    def __init__(self, parameters=None, report_handler=ReportHandler(), formatter=CSVFormatter):
        self._data = []
        self.output = StringIO.StringIO()
        self.formatter = formatter(self)
        self.content = None
        self.asynchronous = getattr(self, "asynch", False)
        self.run = False
        self.report_handler = report_handler
        self.report_handler.add_report(self)
        self._status = None
        self.parameters = parameters or {}

    @classmethod
    def get_verbose_name(self):
        return getattr(self, "verbose_name", self.name)

    def populate(self):
        raise NotImplementedError()

    def _populate(self):
       self.populate()
       self.post_populate()

    def status(self):
        if not self.run:
            return self.NO_RUN
        if self.asynchronous:
            if self._status.ready():
                if self._status.successful():
                    return self.DONE
                else:
                    return self.FAILED
            else:
                return self.RUNNING
        else:
            return self.DONE

    def produce(self, **kwargs):
        self.run = True
        self.report_handler.pre_run(report=self, **kwargs)
        if self.asynchronous:
            self._status = async_populate.delay(self) 
        else:
            self._populate()
        self.report_handler.post_run(report=self, **kwargs)
        return self.status()

    def write_line(self, line):
        self.formatter.write([field.encode(getattr(self, "encoding", "utf8")) if hasattr(field, "encode") else field for field in line])

    def get_data(self):
        for line in self._data:
            self.write_line(line)
        self.content = self.output.getvalue()
        return self.content

    def post_populate(self):
        raise NotImplemented()

    def as_file(self, filename):
        file_ = open(filename, "wb")
        if not self.content:
            self.get_data()
        file_.write(self.content)
        return file_

    @classmethod
    def get_form_class(cls):
        return None

    @classmethod
    def get_form(cls):
        if cls.get_form_class():
            return cls.get_form_class()()
        
@task
def async_populate(instance):
     status = instance._populate()
     instance.post_populate()
     return status
        

class HTSQLReport(Report):
    def __init__(self, *args, **kwargs):
        super(HTSQLReport, self).__init__(*args, **kwargs)
        self._connexion = getattr(self, "connexion", None) or getattr(settings,"HTSQL_REPORT_CONNEXION")
        self._session = HTSQL(self._connexion)

    def populate(self):
        self._data = self._session.produce(self.query, **self.parameters)

    def post_populate(self):
        now_ = datetime.datetime.now().strftime("%Y%M%d_%H%m%s")
        self.filename = "/tmp/%s_%s_%d" % (self.name, now_,  id(self))
        self.as_file(self.filename)



import copy
import shutil

from django.views.generic import ListView, DetailView
from django.http import HttpResponse

from .models import ReportTracking
from .report_handler import DjangoReportHandler
from . import get_report_by_name

# Create your views here.
class ReportTrackingView(ListView):
    model = ReportTracking
    template_name = "report_tracking/reports.html"
    context_object_name = "reports_list"
    paginate_by = 10

    def get(self, request, *args, **kwargs):
       return  super(ReportTrackingView, self).get(request, *args, **kwargs)

    def post(self, request):
        data = copy.copy(request.POST)
        report_type = data.pop('report_action')[0]
        del data['csrfmiddlewaretoken']
        report_class = get_report_by_name(report_type)
        form_class = report_class.get_form_class()
        form = form_class(data)
        if not form.is_valid():
            self.invalid_form = form
            self.invalid_report_name = report_type
        else:
            report = report_class(report_handler=DjangoReportHandler(), parameters=data)
            user = self.request.user if self.request.user.is_authenticated() else None
            report.produce(user=user)
        return self.get(request)

    def get_context_data(self, *args, **kwargs):
        context = super(ReportTrackingView, self).get_context_data(*args, **kwargs)
        if hasattr(self, "invalid_form"):
            context['invalid_form'] = self.invalid_form
            context['invalid_report_name'] = self.invalid_report_name
        return context

class ReportTrackingDetailView(DetailView):
    queryset = ReportTracking.objects.all()

    def get(self, request, *args, **kwargs):
        super(ReportTrackingDetailView, self).get(request, *args, **kwargs)
        response = HttpResponse(mimetype="text/csv")
        new_file = self.object.report_name + ' ' \
                   + str(self.object.report_date) + '.csv'
        response['Content-Disposition'] = 'attachment; filename=%s' % new_file
        file_ = open(self.object.report_file) 
        shutil.copyfileobj(file_, response)
        return response



import csv

class CSVFormatter(object):
    def __init__(self, report):
        self.report = report
        self.delimiter = getattr(self.report, "delimiter", ";")
        self.writer = csv.writer(self.report.output, delimiter=self.delimiter)

    def write(self, line):
        self.writer.writerow(line)
